@@ -5,7 +5,6 @@ from lqs.common.facade import Facade
 from lqs.client.config import RESTClientConfig
 from lqs.client.core import LogQS
 from lqs.client.dsm import DataStoreManager
-from lqs.client.pjm import ProcessJobManager
 from lqs.client.admin import Admin
 from lqs.client.utils import Utils
 
@@ -25,7 +24,6 @@ class RESTClient(Facade):
 
         self.lqs = LogQS(app=self)
         self.dsm = DataStoreManager(app=self)
-        self.pjm = ProcessJobManager(app=self)
         self.admin = Admin(app=self)
 
         self.list = self.lqs.list
