@@ -13,6 +13,7 @@ import urllib.parse
 if TYPE_CHECKING:
     from lqs.client import RESTClient
 
+from lqs import __version__
 from lqs.common import exceptions
 
 
@@ -66,7 +67,7 @@ class RESTInterface:
                 self.app.config.dsm_api_key_id is not None
                 and self.app.config.dsm_api_key_secret is not None
             ):
-                self.app.debug("Using DSM API Key")
+                self.app.logger.debug("Using DSM API Key")
                 api_key_id = self.app.config.dsm_api_key_id
                 api_key_secret = self.app.config.dsm_api_key_secret
 
@@ -85,8 +86,13 @@ class RESTInterface:
         url_params = ""
         for key, value in args.items():
             if value is not None and key not in ["self"] + exclude:
-                if type(value) == dict or type(value) == list:
+                if type(value) is dict:
                     value = json.dumps(value, cls=LogQSEncoder)
+                elif type(value) is list:
+                    if all(isinstance(i, (str, int, float, UUID)) for i in value):
+                        for i in value:
+                            url_params += f"&{key}={i}"
+                        continue
                 quoted_value = urllib.parse.quote(str(value))
                 url_params += f"&{key}={quoted_value}"
         if len(url_params) > 0:
@@ -100,7 +106,47 @@ class RESTInterface:
                 payload[key] = value
         return payload
 
+    def _check_version_match(
+        self, api_version: Optional[str], client_version: Optional[str]
+    ) -> bool:
+        if self.app.config.version_match_level is None:
+            return True
+        if api_version is None:
+            return True
+        if client_version is None:
+            return True
+
+        api_version_parts = api_version.split(".")
+        client_version_parts = client_version.split(".")
+
+        if self.app.config.version_match_level.lower() == "major":
+            return api_version_parts[0] == client_version_parts[0]
+        elif self.app.config.version_match_level.lower() == "minor":
+            return (
+                api_version_parts[0] == client_version_parts[0]
+                and api_version_parts[1] == client_version_parts[1]
+            )
+        elif self.app.config.version_match_level.lower() == "patch":
+            return (
+                api_version_parts[0] == client_version_parts[0]
+                and api_version_parts[1] == client_version_parts[1]
+                and api_version_parts[2] == client_version_parts[2]
+            )
+
     def _handle_response_data(self, response: requests.Response):
+        api_version = response.headers.get("x-logqs-version")
+        if api_version is not None:
+            versions_match = self._check_version_match(
+                api_version=api_version,
+                client_version=__version__,
+            )
+            if not versions_match:
+                version_mismatch_message = f"Version mismatch: client version: {__version__}, server version: {api_version}. Update the client to match the server version."
+                if self.app.config.raise_for_version_mismatch:
+                    raise Exception(version_mismatch_message)
+                if self.app.config.warn_for_version_mismatch:
+                    self.app.logger.warning(version_mismatch_message)
+
         if response.status_code == 204:
             return
 
@@ -125,8 +171,13 @@ class RESTInterface:
             return response_data
         else:
             response_message = response_data
+            if isinstance(response_data, bytes):
+                raise Exception(f"Error: {response_data}")
+
             if "message" in response_data:
                 response_message = response_data["message"]
+            if response.status_code == 400:
+                raise exceptions.BadRequestException(msg=response_message)
             if response.status_code == 401:
                 raise exceptions.UnauthorizedException(msg=response_message)
             elif response.status_code == 403:
@@ -143,6 +194,8 @@ class RESTInterface:
                 raise exceptions.LockedException(msg=response_message)
             elif response.status_code == 500:
                 raise exceptions.InternalServerErrorException(msg=response_message)
+            elif response.status_code == 501:
+                raise exceptions.NotImplementedException(msg=response_message)
             raise Exception(response_message)
 
     def _handle_retries(self, func, retry_count=None):
@@ -162,6 +215,7 @@ class RESTInterface:
                         exceptions.ConflictException,
                         exceptions.ContentTooLargeException,
                         exceptions.LockedException,
+                        exceptions.BadRequestException,
                     ]
                     for exception in lqs_expected_exceptions:
                         if isinstance(e, exception):
@@ -169,8 +223,8 @@ class RESTInterface:
                 if retry_count > 0 and i < retry_count:
                     # exponential backoff
                     backoff = self.app.config.retry_delay * (2**i)
-                    self.app.error(f"Error: {e}")
-                    self.app.info(f"Retrying in {backoff} seconds")
+                    self.app.logger.error(f"Error: {e}")
+                    self.app.logger.debug(f"Retrying in {backoff} seconds")
                     time.sleep(backoff)
                 else:
                     raise e
@@ -179,7 +233,7 @@ class RESTInterface:
     def _get_url(self, resource_path):
         path_prefix = f"/{self.service}/api"
         datastore_id = self.app.get_datastore_id()
-        if self.service in ("lqs", "pjm") and datastore_id is not None:
+        if self.service == "lqs" and datastore_id is not None:
             path_prefix = f"/{self.service}/{datastore_id}/api"
 
         url = self.app.config.api_url or ""
@@ -191,7 +245,13 @@ class RESTInterface:
     def _head_resource(self, resource_path):
         url = self._get_url(resource_path)
         if self.app.config.dry_run:
-            self.app.info(f"(Dry Run) HEAD {url}")
+            self.app.logger.info(
+                {
+                    "log_type": "dry_run",
+                    "method": "HEAD",
+                    "url": url,
+                }
+            )
             return {}
 
         def make_request():
@@ -217,7 +277,15 @@ class RESTInterface:
     ):
         url = self._get_url(resource_path)
         if self.app.config.dry_run:
-            self.app.info(f"(Dry Run) GET {url}")
+            self.app.logger.info(
+                {
+                    "log_type": "dry_run",
+                    "method": "GET",
+                    "url": url,
+                    "expected_content_type": expected_content_type,
+                    "additional_headers": additional_headers,
+                }
+            )
             return {}
 
         def make_request():
@@ -257,7 +325,16 @@ class RESTInterface:
     ):
         url = self._get_url(resource_path)
         if self.app.config.dry_run:
-            self.app.info(f"(Dry Run) POST {url}, {data}")
+            self.app.logger.info(
+                {
+                    "log_type": "dry_run",
+                    "method": "POST",
+                    "url": url,
+                    "data": data,
+                    "response_model": response_model,
+                    "additiona_params": additiona_params,
+                }
+            )
             return {}
 
         def make_request():
@@ -294,7 +371,16 @@ class RESTInterface:
     ):
         url = self._get_url(resource_path)
         if self.app.config.dry_run:
-            self.app.info(f"(Dry Run) PATCH {url}, {data}")
+            self.app.logger.info(
+                {
+                    "log_type": "dry_run",
+                    "method": "PATCH",
+                    "url": url,
+                    "data": data,
+                    "response_model": response_model,
+                    "additiona_params": additiona_params,
+                }
+            )
             return {}
 
         def make_request():
@@ -326,7 +412,14 @@ class RESTInterface:
     def _delete_resource(self, resource_path, additiona_params={}):
         url = self._get_url(resource_path)
         if self.app.config.dry_run:
-            self.app.info(f"(Dry Run) DELETE {url}")
+            self.app.logger.info(
+                {
+                    "log_type": "dry_run",
+                    "method": "DELETE",
+                    "url": url,
+                    "additiona_params": additiona_params,
+                }
+            )
             return
 
         def make_request():
