@@ -4,9 +4,12 @@ import inspect
 import logging
 import random
 import time
+import base64
+import struct
 from functools import partial
 from typing import Optional, Union
 
+import av
 import s3path
 import bz2
 import lz4.frame
@@ -16,12 +19,14 @@ import numpy as np
 import DracoPy as draco
 from pythonjsonlogger import jsonlogger
 from PIL import Image as ImagePIL
+from PIL import ImageDraw
 from mcap.data_stream import ReadDataStream
 from mcap.records import Chunk
 
 
 class ContextLoggerAdapter(logging.LoggerAdapter):
     def process(self, msg, kwargs):
+        kwargs["extra"] = self.extra
         return msg, kwargs
 
     def log(self, level, msg, *args, **kwargs):
@@ -46,7 +51,7 @@ class ContextLoggerAdapter(logging.LoggerAdapter):
                     args,
                     None,
                     func_name,
-                    None,
+                    extra=self.extra,
                 )
                 self.logger.handle(record)
 
@@ -56,6 +61,7 @@ def get_logger(
     level: Optional[str] = None,
     log_to_file: bool = False,
     json_logging: bool = False,
+    correlation_id: str | None = None,
 ):
     log_level = level or os.environ.get("LOG_LEVEL", "INFO")
     if isinstance(log_level, str):
@@ -64,19 +70,19 @@ def get_logger(
     logger.setLevel(log_level)
     # logger.propagate = False
     if logger.hasHandlers():
-        return logger
-
-    formatter = logging.Formatter(
-        "%(asctime)s  (%(levelname)s - %(name)s): %(message)s"
-    )
+        return ContextLoggerAdapter(logger, {"correlation_id": correlation_id})
+    correlation_id_str = "%(correlation_id)s:" if correlation_id else ""
     if json_logging:
         formatter = jsonlogger.JsonFormatter(
-            "%(asctime)s %(levelname)s %(name)s %(filename)s %(funcName)s %(lineno)s %(message)s"
+            f"%(asctime)s %(levelname)s {correlation_id_str} %(name)s %(filename)s %(funcName)s %(lineno)s %(message)s"
         )
         handler = logging.StreamHandler()
         handler.setFormatter(formatter)
         logger.addHandler(handler)
     else:
+        formatter = logging.Formatter(
+            f"%(asctime)s  (%(levelname)s - %(name)s): {correlation_id_str} %(message)s"
+        )
         handler = logging.StreamHandler()
         handler.setFormatter(formatter)
         logger.addHandler(handler)
@@ -85,8 +91,7 @@ def get_logger(
             handler = logging.FileHandler("lqs.log")
             handler.setFormatter(formatter)
             logger.addHandler(handler)
-
-    return ContextLoggerAdapter(logger, {})
+    return ContextLoggerAdapter(logger, {"correlation_id": correlation_id})
 
 
 def attempt_with_retries(
@@ -157,6 +162,8 @@ def decompress_chunk_bytes(chunk_bytes, chunk_compression, chunk_length=None):
         decompressed_bytes = lz4.block.decompress(chunk_bytes, chunk_length)
     elif chunk_compression in [None, "none", ""]:
         decompressed_bytes = chunk_bytes
+    elif chunk_compression == "mp4":
+        decompressed_bytes = chunk_bytes
     else:
         raise ValueError(f"Unknown chunk compression type: {chunk_compression}")
     return decompressed_bytes
@@ -164,8 +171,203 @@ def decompress_chunk_bytes(chunk_bytes, chunk_compression, chunk_length=None):
 
 # Image
 
+image_type_names = [
+    "sensor_msgs/Image",
+    "sensor_msgs/CompressedImage",
+    "stereo_msgs/DisparityImage",
+    "sensor_msgs/msg/Image",
+    "sensor_msgs/msg/CompressedImage",
+    "stereo_msgs/msg/DisparityImage",
+    "ark::image::Image",
+    "logqs.image",
+    "logqs.inference.depth-estimation",
+    "logqs.inference.image-to-image",
+    "lqs_cutter/navcon_image",
+]
 
-def get_record_image_ros(record_data) -> ImagePIL.Image:
+
+def image_to_base64(
+    image: ImagePIL.Image, format: str = "WEBP", format_params: dict = {}
+) -> str:
+    buffered_img = io.BytesIO()
+    image.save(buffered_img, format=format, **format_params)
+    return base64.b64encode(buffered_img.getvalue()).decode("utf-8")
+
+
+def get_complete_segmentation_image(output, meta):
+    image_size = (
+        meta["image"]["width"],
+        meta["image"]["height"],
+    )
+    img = ImagePIL.new("RGB", image_size, (0, 0, 0))
+    for segmentation in output:
+        mask_data = base64.b64decode(segmentation["mask"])
+        mask = ImagePIL.open(io.BytesIO(mask_data)).convert("L")
+
+        label = segmentation.get("label")
+        if label is None:
+            color = (
+                random.randint(0, 255),
+                random.randint(0, 255),
+                random.randint(0, 255),
+            )
+        else:
+            color = int(hash(label) % 0xFFFFFF)
+            color = (
+                color & 0xFF,
+                (color >> 8) & 0xFF,
+                (color >> 16) & 0xFF,
+            )
+        draw = ImageDraw.Draw(img)
+        draw.bitmap((0, 0), mask, fill=color)
+    return img
+
+
+def convert_avc1_to_annexb(data, filter_parameter_sets=True):
+    # Wrap data in a memoryview and cache its length.
+    mv = memoryview(data)
+    mv_len = len(mv)
+    offset = 0
+    annexb = bytearray()
+    start_code = b"\x00\x00\x00\x01"
+
+    while offset + 4 <= mv_len:
+        nal_length = struct.unpack_from(">I", mv, offset)[0]
+        offset += 4
+        # Validate NAL length.
+        if nal_length <= 0 or offset + nal_length > mv_len:
+            break
+        # Avoid parameter set duplication.
+        nal_unit = mv[offset : offset + nal_length].tobytes()
+        offset += nal_length
+        nal_type = nal_unit[0] & 0x1F
+        if filter_parameter_sets and nal_type in (7, 8):
+            continue
+        annexb.extend(start_code)
+        annexb.extend(nal_unit)
+    return bytes(annexb)
+
+
+def convert_hvc1_to_annexb(data):
+    mv = memoryview(data)
+    total_len = len(mv)
+    offset = 0
+    result = bytearray()
+    start_code = b"\x00\x00\x00\x01"
+
+    while offset + 4 <= total_len:
+        nal_length = struct.unpack_from(">I", mv, offset)[0]
+        offset += 4
+        if nal_length <= 0 or offset + nal_length > total_len:
+            break
+        result.extend(start_code)
+        result.extend(mv[offset : offset + nal_length])
+        offset += nal_length
+    return bytes(result)
+
+
+def generate_image_from_frames(extradata, frames, frame_data, encoding):
+    # Create the decoder and set extradata.
+    codec = av.codec.CodecContext.create(encoding, "r")
+    codec.extradata = extradata
+
+    # Precompute start_offset and wrap frame_data in a memoryview.
+    start_offset = frames[0][2]
+    fd_mv = memoryview(frame_data)
+    fd_len = len(fd_mv)
+    converter = convert_hvc1_to_annexb if encoding == "hevc" else convert_avc1_to_annexb
+
+    decoded_frames = None
+    for frame in frames:
+        # Compute slice boundaries once.
+        offset_in_frame = frame[2] - start_offset
+        size = frame[1]
+        # Check bounds.
+        if fd_len < offset_in_frame + size:
+            break
+        # Slice out the NAL unit data.
+        nal_data = fd_mv[offset_in_frame : offset_in_frame + size]
+        # Call the converter; if nal_data is already a memoryview, let the converter wrap it.
+        frame_annexb = converter(nal_data)
+        packet = av.Packet(frame_annexb)
+        decoded_frames = codec.decode(packet)
+    # We expect a single decoded frame.
+    assert (
+        decoded_frames is not None and len(decoded_frames) == 1
+    ), f"Delta frame decoding failed: {len(decoded_frames) if decoded_frames else 0} frames decoded, expected 1"
+    decoded_frame = decoded_frames[0]
+    image = decoded_frame.to_image()
+    return image
+
+
+def get_record_image_lqs(record_data, **kwargs) -> ImagePIL.Image:
+    if "frames" in kwargs:
+        # if we've passed in frame information, then this is a video frame
+        frames = kwargs["frames"]
+        encoding = kwargs["encoding"]
+
+        extradata = kwargs.get("extradata", None)
+        if isinstance(extradata, str):
+            extradata = base64.b64decode(extradata)
+
+        frame_data = record_data
+        if isinstance(frame_data, str):
+            frame_data = base64.b64decode(frame_data)
+
+        img = generate_image_from_frames(extradata, frames, frame_data, encoding)
+    elif isinstance(record_data, str):
+        # if the record data is a string, it is a base64 encoded image
+        try:
+            image_data = base64.b64decode(record_data)
+        except Exception as e:
+            raise Exception(f"Failed to decode record data: {e}")
+        try:
+            img = ImagePIL.open(io.BytesIO(image_data))
+        except Exception as e:
+            raise Exception(f"Failed to open image: {e}")
+    else:
+        meta = record_data.get("meta")
+        if meta:
+            pipeline = meta.get("pipeline")
+            if pipeline:
+                task = pipeline.get("task")
+                if task is None:
+                    raise Exception(
+                        "Expected to find 'task' in record data's 'meta.pipeline' field."
+                    )
+                if task == "depth-estimation":
+                    image_data = base64.b64decode(record_data["output"]["depth"])
+                    img = ImagePIL.open(io.BytesIO(image_data))
+                elif task == "image-segmentation":
+                    img = get_complete_segmentation_image(
+                        output=record_data["output"], meta=meta
+                    )
+                elif task == "image-to-image":
+                    image_data = base64.b64decode(record_data["output"])
+                    img = ImagePIL.open(io.BytesIO(image_data))
+                else:
+                    raise Exception(
+                        f"Task '{task}' not supported for image generation."
+                    )
+            else:
+                raise Exception(
+                    "Expected to find 'pipeline' in record data's 'meta' field."
+                )
+        else:
+            raise Exception("Expected to find 'meta' in record data.")
+    return img
+
+
+def normalize_disparity_image(
+    image: ImagePIL.Image, min_clip=74, max_clip=2592, **kwargs
+):
+    pixels = np.asarray(image)
+    pixels = np.clip(pixels, min_clip, max_clip)
+    pixels = ((pixels - min_clip) / (max_clip - min_clip)) * 255.0
+    return ImagePIL.fromarray(pixels).convert("L")
+
+
+def get_record_image_ros(record_data, renormalize=True, **kwargs) -> ImagePIL.Image:
     img_modes = {
         "16UC1": "I;16",
         "mono8": "L",
@@ -208,10 +410,12 @@ def get_record_image_ros(record_data) -> ImagePIL.Image:
         if encoding == "bgr8":
             b, g, r = img.split()
             img = ImagePIL.merge("RGB", (r, g, b))
-
-        if encoding in ["mono16", "16UC1", "32FC1"]:
+        elif encoding in ["mono16", "16UC1", "32FC1"] and renormalize:
+            if kwargs.get("normalize_disparity"):
+                img = normalize_disparity_image(img, **kwargs)
             pixels = np.asarray(img)
-            pixel_range = np.max(pixels) - np.min(pixels)
+            pixel_max = np.max(pixels)
+            pixel_range = pixel_max - np.min(pixels)
             if pixel_range == 0:
                 pixels = np.zeros_like(pixels)
             else:
@@ -230,7 +434,7 @@ def get_record_image_ros(record_data) -> ImagePIL.Image:
     return img
 
 
-def get_record_image_ark(record_data) -> ImagePIL.Image:
+def get_record_image_ark(record_data, renormalize=True, **kwargs) -> ImagePIL.Image:
     data = bytes(record_data["data"])
     image_format = record_data["data_format"]
     image_format_mapping = {
@@ -261,10 +465,11 @@ def get_record_image_ark(record_data) -> ImagePIL.Image:
 
     if image_format == "depthz16":
         ark_img = np.frombuffer(data, dtype=np.float16)
-        pixel_range = np.max(ark_img) - np.min(ark_img)
 
-        if pixel_range != 0:
-            ark_img = ((ark_img - np.min(ark_img)) / pixel_range) * 65535.0
+        if renormalize:
+            pixel_range = np.max(ark_img) - np.min(ark_img)
+            if pixel_range != 0:
+                ark_img = ((ark_img - np.min(ark_img)) / pixel_range) * 65535.0
 
         ark_img = ark_img.astype(np.uint16)
         data = ark_img
@@ -373,23 +578,15 @@ def get_record_image_ark(record_data) -> ImagePIL.Image:
     return img
 
 
-# deprecated, remove this later
-def get_image_from_ark_json_msg(
-    record_data,
-    max_size: Optional[int] = None,
-    format: str = "WEBP",
-    format_params: dict = {},
-    reset_position: bool = True,
-    return_bytes: bool = False,
-):
-    return get_record_image(
-        record_data=record_data,
-        max_size=max_size,
-        format=format,
-        format_params=format_params,
-        reset_position=reset_position,
-        return_bytes=return_bytes,
-    )
+def get_record_image_cutter(record_data, renormalize=True, **kwargs) -> ImagePIL.Image:
+    image_format = record_data["type"]
+    images_data = record_data["data"][0]
+    if image_format != "JPG":
+        raise NotImplementedError(
+            f"Message type {image_format} not supported for image conversion."
+        )
+    img = ImagePIL.open(io.BytesIO(bytes(images_data)))
+    return img
 
 
 def get_record_image(
@@ -397,32 +594,54 @@ def get_record_image(
     max_size: Optional[int] = None,
     format: str = "WEBP",
     format_params: dict = {},
+    renormalize: bool = True,
+    resample=ImagePIL.Resampling.NEAREST,
     reset_position: bool = True,
     return_bytes: bool = False,
+    **kwargs,
 ) -> Union[ImagePIL.Image, io.BytesIO, None]:
-    _format = record_data.get("format", None)
-    _encoding = record_data.get("encoding", None)
-    _data_format = record_data.get("data_format", None)
-    _image = record_data.get("image", None)
-    if _encoding is not None:
-        # ROS sensor_msgs/Image
-        img = get_record_image_ros(record_data=record_data)
-    elif _format is not None:
-        # ROS sensor_msgs/CompressedImage
-        img = get_record_image_ros(record_data=record_data)
-    elif _data_format is not None:
-        # ARK ark::image::Image
-        img = get_record_image_ark(record_data=record_data)
-    elif _image is not None:
-        # ROS stereo_msgs/DisparityImage
-        img = get_record_image_ros(record_data=_image)
+    if isinstance(record_data, bytes):
+        img = get_record_image_lqs(
+            record_data=record_data, renormalize=renormalize, **kwargs
+        )
     else:
-        # we don't know what it is, but we'll try ROS
-        img = get_record_image_ros(record_data=record_data)
+        _format = record_data.get("format", None)
+        _encoding = record_data.get("encoding", None)
+        _data_format = record_data.get("data_format", None)
+        _image = record_data.get("image", None)
+        if record_data.get("type") and record_data.get("cameras"):
+            img = get_record_image_cutter(
+                record_data=record_data, renormalize=renormalize, **kwargs
+            )
+        elif _encoding is not None:
+            # ROS sensor_msgs/Image
+            img = get_record_image_ros(
+                record_data=record_data, renormalize=renormalize, **kwargs
+            )
+        elif _format is not None:
+            # ROS sensor_msgs/CompressedImage
+            img = get_record_image_ros(
+                record_data=record_data, renormalize=renormalize, **kwargs
+            )
+        elif _data_format is not None:
+            # ARK ark::image::Image
+            img = get_record_image_ark(
+                record_data=record_data, renormalize=renormalize, **kwargs
+            )
+        elif _image is not None:
+            # ROS stereo_msgs/DisparityImage
+            img = get_record_image_ros(
+                record_data=_image, renormalize=renormalize, **kwargs
+            )
+        else:
+            # we default to LQS image formats
+            img = get_record_image_lqs(
+                record_data=record_data, renormalize=renormalize, **kwargs
+            )
 
     if max_size is not None:
         if img.width > max_size or img.height > max_size:
-            img.thumbnail((max_size, max_size))
+            img.thumbnail((max_size, max_size), resample=resample)
 
     if return_bytes:
         buffered_img = io.BytesIO()
@@ -434,7 +653,12 @@ def get_record_image(
         return img
 
 
-# PointCloud
+# Point Cloud
+
+point_cloud_type_names = [
+    "sensor_msgs/PointCloud2",
+    "sensor_msgs/msg/PointCloud2",
+]
 
 
 def jet_colormap(values, normalize=True):
@@ -494,7 +718,9 @@ def process_pointcloud_record_data(
     max_points=None,
     random_sample=True,
     voxel_size=10.0,
-    use_colormap=True,
+    use_colormap=False,
+    max_radius=None,
+    **kwargs,
 ):
     fields = record_data["fields"]
     assert [f["name"] for f in fields[:3]] == [
@@ -514,7 +740,24 @@ def process_pointcloud_record_data(
         7: np.float32,
         8: np.float64,
     }
-    dtype = np.dtype([(f["name"], dtype_map[f["datatype"]]) for f in fields])
+    dtype_list = []
+    current_offset = 0
+    for field in fields:
+        field_name = field["name"]
+        field_type = dtype_map[field["datatype"]]
+        field_offset = field["offset"]
+        # Check if there is padding needed
+        if field_offset > current_offset:
+            padding_size = field_offset - current_offset
+            dtype_list.append(("padding" + str(current_offset), np.uint8, padding_size))
+        dtype_list.append((field_name, field_type))
+        current_offset = field_offset + np.dtype(field_type).itemsize
+
+    if current_offset < record_data["point_step"]:
+        padding_size = record_data["point_step"] - current_offset
+        dtype_list.append(("padding" + str(current_offset), np.uint8, padding_size))
+
+    dtype = np.dtype(dtype_list)
     all_data = np.frombuffer(point_cloud_data, dtype=dtype)
 
     # Access positions directly via field names, assuming the first three fields are x, y, z
@@ -532,14 +775,20 @@ def process_pointcloud_record_data(
         colors = np.vstack([red, green, blue]).T.astype(np.uint8)
     elif len(fields) > 3 and fields[3]["name"] == "intensity":
         # Process intensity data if needed
-        colors = all_data[
-            "intensity"
-        ]  # Assuming 'intensity' directly gives the desired data
+        colors = all_data["intensity"].astype(
+            np.uint8
+        )  # Assuming 'intensity' directly gives the desired data
         if use_colormap:
             colors = jet_colormap(colors)
         else:
             colors = np.stack([colors] * 3, axis=-1)
 
+    if max_radius is not None:
+        indices = np.linalg.norm(positions, axis=1) < max_radius
+        positions = positions[indices]
+        if colors is not None:
+            colors = colors[indices]
+
     if max_points and len(positions) > max_points:
         if random_sample:
             indices = np.random.choice(len(positions), max_points, replace=False)
@@ -562,6 +811,9 @@ def encode_to_draco(
     colors=None,
     max_points=None,
 ):
+    if len(positions) == 0:
+        raise ValueError("Empty positions array for encoding to Draco.")
+
     draco_binary = draco.encode(
         positions,
         faces=None,
@@ -599,6 +851,10 @@ def encode_to_draco(
     return draco_binary
 
 
+def decode_draco(draco_binary):
+    return draco.decode(draco_binary).points, draco.decode(draco_binary).colors
+
+
 # MCAP
 
 
