@@ -1,4 +1,4 @@
-from typing import Optional
+from typing import Optional, List, Union
 from datetime import datetime
 from uuid import UUID
 from abc import abstractmethod
@@ -25,7 +25,7 @@ class ListInterface(BaseListInterface):
         progress_null: Optional[bool] = None,
         progress_gte: Optional[float] = None,
         progress_lte: Optional[float] = None,
-        error_like: Optional[str] = None,
+        error_filter: Optional[str] = None,
         note_like: Optional[str] = None,
         context_filter: Optional[str] = None,
         workflow_context_filter: Optional[str] = None,
@@ -63,7 +63,7 @@ class ListInterface(BaseListInterface):
             progress_null=progress_null,
             progress_gte=progress_gte,
             progress_lte=progress_lte,
-            error_like=error_like,
+            error_filter=error_filter,
             note_like=note_like,
             context_filter=context_filter,
             workflow_context_filter=workflow_context_filter,
@@ -389,7 +389,7 @@ class ListInterface(BaseListInterface):
         progress_null: Optional[bool] = None,
         progress_gte: Optional[float] = None,
         progress_lte: Optional[float] = None,
-        error_like: Optional[str] = None,
+        error_filter: Optional[str] = None,
         note_like: Optional[str] = None,
         context_filter: Optional[str] = None,
         include_count: Optional[bool] = True,
@@ -430,7 +430,7 @@ class ListInterface(BaseListInterface):
             progress_null=progress_null,
             progress_gte=progress_gte,
             progress_lte=progress_lte,
-            error_like=error_like,
+            error_filter=error_filter,
             note_like=note_like,
             context_filter=context_filter,
             include_count=include_count,
@@ -601,9 +601,6 @@ class ListInterface(BaseListInterface):
         locked: Optional[bool] = None,
         note_like: Optional[str] = None,
         context_filter: Optional[str] = None,
-        time_adjustment_null: Optional[bool] = None,
-        time_adjustment_gte: Optional[models.Int64] = None,
-        time_adjustment_lte: Optional[models.Int64] = None,
         start_time_null: Optional[bool] = None,
         start_time_gte: Optional[models.Int64] = None,
         start_time_lte: Optional[models.Int64] = None,
@@ -618,6 +615,8 @@ class ListInterface(BaseListInterface):
         object_size_lte: Optional[int] = None,
         object_count_gte: Optional[int] = None,
         object_count_lte: Optional[int] = None,
+        group_id_in: Optional[Union[UUID, List[UUID]]] = None,
+        tag_label_ids_includes: Optional[Union[UUID, List[UUID]]] = None,
         include_count: Optional[bool] = True,
         offset: Optional[int] = 0,
         limit: Optional[int] = 100,
@@ -650,9 +649,6 @@ class ListInterface(BaseListInterface):
             locked=locked,
             note_like=note_like,
             context_filter=context_filter,
-            time_adjustment_null=time_adjustment_null,
-            time_adjustment_gte=time_adjustment_gte,
-            time_adjustment_lte=time_adjustment_lte,
             start_time_null=start_time_null,
             start_time_gte=start_time_gte,
             start_time_lte=start_time_lte,
@@ -667,6 +663,8 @@ class ListInterface(BaseListInterface):
             object_size_lte=object_size_lte,
             object_count_gte=object_count_gte,
             object_count_lte=object_count_lte,
+            group_id_in=group_id_in,
+            tag_label_ids_includes=tag_label_ids_includes,
             include_count=include_count,
             offset=offset,
             limit=limit,
@@ -806,6 +804,7 @@ class ListInterface(BaseListInterface):
         self,
         id: Optional[UUID] = None,
         bucket_name: Optional[str] = None,
+        bucket_name_like: Optional[str] = None,
         access_key_id: Optional[str] = None,
         region_name: Optional[str] = None,
         endpoint_url: Optional[str] = None,
@@ -837,6 +836,7 @@ class ListInterface(BaseListInterface):
         return self._object_store(
             id=id,
             bucket_name=bucket_name,
+            bucket_name_like=bucket_name_like,
             access_key_id=access_key_id,
             region_name=region_name,
             endpoint_url=endpoint_url,
@@ -952,7 +952,7 @@ class ListInterface(BaseListInterface):
         ingestion_id: Optional[UUID] = None,
         workflow_id: Optional[UUID] = None,
         workflow_id_null: Optional[bool] = None,
-        error_like: Optional[str] = None,
+        error_filter: Optional[str] = None,
         note_like: Optional[str] = None,
         source: Optional[str] = None,
         query_data_filter: Optional[str] = None,
@@ -1003,7 +1003,7 @@ class ListInterface(BaseListInterface):
             ingestion_id=ingestion_id,
             workflow_id=workflow_id,
             workflow_id_null=workflow_id_null,
-            error_like=error_like,
+            error_filter=error_filter,
             note_like=note_like,
             source=source,
             query_data_filter=query_data_filter,
