@@ -1,8 +1,6 @@
 from typing import List, Optional
 from uuid import UUID
 
-from pydantic import BaseModel
-
 from lqs.interface.core.models.__common__ import (
     CommonModel,
     DataResponseModel,
@@ -15,14 +13,14 @@ from lqs.interface.core.models.__common__ import (
 from lqs.interface.core.models import Log
 
 
-class Group(CommonModel["Group"], LockModel):
+class Group(CommonModel, LockModel):
     name: str
     note: Optional[str]
     context: Optional[dict]
     default_workflow_id: Optional[UUID]
 
     def list_logs(self, **kwargs) -> List["Log"]:
-        return self.app.list.logs(group_id=self.id, **kwargs).data
+        return self._app.list.logs(group_id=self.id, **kwargs).data
 
 
 class GroupDataResponse(DataResponseModel[Group]):
