@@ -1,4 +1,4 @@
-from typing import List, Optional
+from typing import Optional
 from uuid import UUID
 
 from pydantic import BaseModel
@@ -13,7 +13,7 @@ from lqs.interface.core.models.__common__ import (
 )
 
 
-class Hook(CommonModel["Hook"]):
+class Hook(CommonModel):
     workflow_id: UUID
     trigger_process: ProcessType
     trigger_state: ProcessState
@@ -30,7 +30,7 @@ class HookDataResponse(DataResponseModel[Hook]):
 
 
 class HookListResponse(PaginationModel[Hook]):
-	pass
+    pass
 
 
 class HookCreateRequest(BaseModel):
