@@ -1,10 +1,10 @@
-from typing import List, Optional, Tuple
+from typing import List, Optional, Tuple, Union
 from uuid import UUID
 
 from pydantic import BaseModel
 
 from lqs.interface.core.models.__common__ import (
-     DataResponseModel,
+    DataResponseModel,
     PaginationModel,
     ProcessModel,
     ProcessCreateRequest,
@@ -22,6 +22,7 @@ class IngestionPartIndexEntry(BaseModel):
     chunk_offset: Optional[int]
     chunk_length: Optional[int]
     timestamp: Int64
+    # context: Optional[dict] = None
 
 
 # TODO: in Python 3.11, we can use Tuple[*get_type_hints(IngestionPartIndex).values()]
@@ -29,19 +30,29 @@ IngestionPartIndexTuple = Tuple[
     str, int, int, Optional[str], Optional[int], Optional[int], Int64
 ]
 
+IngestionPartIndexTupleV2 = Tuple[
+    str, int, int, Optional[str], Optional[int], Optional[int], Int64, Optional[dict]
+]
+
 
 class IngestionPartIndex(BaseModel):
     ingestion_part_id: UUID
-    index: Optional[List[IngestionPartIndexTuple]]
+    index: Optional[
+        Union[List[IngestionPartIndexTuple], List[IngestionPartIndexTupleV2]]
+    ]
 
 
 class IngestionPartIndexCreateRequest(BaseModel):
     ingestion_part_id: UUID
-    index: Optional[List[IngestionPartIndexTuple]] = None
+    index: Optional[
+        Union[List[IngestionPartIndexTuple], List[IngestionPartIndexTupleV2]]
+    ] = None
 
 
 class IngestionPartIndexUpdateRequest(BaseModel):
-    index: Optional[List[IngestionPartIndexTuple]] = optional_field
+    index: Optional[
+        Union[List[IngestionPartIndexTuple], List[IngestionPartIndexTupleV2]]
+    ] = optional_field
 
 
 class IngestionPartIndexDataResponse(DataResponseModel[IngestionPartIndex]):
@@ -49,7 +60,7 @@ class IngestionPartIndexDataResponse(DataResponseModel[IngestionPartIndex]):
 
 
 class IngestionPartIndexListResponse(PaginationModel[IngestionPartIndex]):
-	pass
+    pass
 
 
 # Ingestion Part
@@ -59,26 +70,32 @@ class IngestionPart(ProcessModel["IngestionPart"]):
     sequence: int
     ingestion_id: UUID
     source: Optional[str]
-    index: Optional[List[IngestionPartIndexTuple]]
+    index: Optional[
+        Union[List[IngestionPartIndexTuple], List[IngestionPartIndexTupleV2]]
+    ]
 
-    log_id: Optional[UUID] = None
-    group_id: Optional[UUID] = None
+    log_id: Optional[UUID]
+    group_id: Optional[UUID]
 
 
 class IngestionPartCreateRequest(ProcessCreateRequest):
     sequence: int
     source: Optional[str] = None
-    index: Optional[List[IngestionPartIndexTuple]] = None
+    index: Optional[
+        Union[List[IngestionPartIndexTuple], List[IngestionPartIndexTupleV2]]
+    ] = None
 
 
 class IngestionPartUpdateRequest(ProcessUpdateRequest):
     sequence: int = optional_field
     source: Optional[str] = optional_field
-    index: Optional[List[IngestionPartIndexTuple]] = optional_field
+    index: Optional[
+        Union[List[IngestionPartIndexTuple], List[IngestionPartIndexTupleV2]]
+    ] = optional_field
 
 
 class IngestionPartListResponse(PaginationModel[IngestionPart]):
-	pass
+    pass
 
 
 class IngestionPartDataResponse(DataResponseModel[IngestionPart]):
