@@ -1,4 +1,4 @@
-from typing import List, Optional
+from typing import Optional
 
 from pydantic import BaseModel
 
@@ -10,7 +10,7 @@ from lqs.interface.core.models.__common__ import (
 )
 
 
-class Label(CommonModel["Label"]):
+class Label(CommonModel):
     value: str
     note: Optional[str]
 
@@ -20,7 +20,7 @@ class LabelDataResponse(DataResponseModel[Label]):
 
 
 class LabelListResponse(PaginationModel[Label]):
-	pass
+    pass
 
 
 class LabelCreateRequest(BaseModel):
