@@ -11,7 +11,7 @@ from lqs.interface.core.models.__common__ import (
 )
 
 
-class Query(CommonModel["Query"]):
+class Query(CommonModel):
     log_id: UUID
     name: Optional[str]
     note: Optional[str]
@@ -29,7 +29,7 @@ class QueryDataResponse(DataResponseModel[Query]):
 
 
 class QueryListResponse(PaginationModel[Query]):
-	pass
+    pass
 
 
 class QueryCreateRequest(BaseModel):
