@@ -1,4 +1,4 @@
-from typing import List, Optional
+from typing import Optional
 from uuid import UUID
 
 from pydantic import BaseModel
@@ -12,7 +12,7 @@ from lqs.interface.core.models.__common__ import (
 )
 
 
-class Tag(CommonModel["Tag"]):
+class Tag(CommonModel):
     label_id: UUID
     log_id: UUID
     topic_id: Optional[UUID]
@@ -28,7 +28,7 @@ class TagDataResponse(DataResponseModel[Tag]):
 
 
 class TagListResponse(PaginationModel[Tag]):
-	pass
+    pass
 
 
 class TagCreateRequest(BaseModel):
