@@ -1,4 +1,4 @@
-from typing import List, Optional
+from typing import Optional
 from uuid import UUID
 
 from pydantic import BaseModel
@@ -28,7 +28,7 @@ class CommentDataResponse(DataResponseModel[Comment]):
 
 
 class CommentListResponse(PaginationModel[Comment]):
-	pass
+    pass
 
 
 class CommentCreateRequest(BaseModel):
