@@ -1,4 +1,4 @@
-from typing import List, Optional
+from typing import Optional
 
 from pydantic import BaseModel
 
@@ -23,7 +23,7 @@ class ConfigurationDataResponse(DataResponseModel[Configuration]):
 
 
 class ConfigurationListResponse(PaginationModel[Configuration]):
-	pass
+    pass
 
 
 class ConfigurationCreateRequest(BaseModel):
