@@ -1,5 +1,5 @@
 from uuid import UUID
-from typing import List, Optional
+from typing import Optional
 
 from pydantic import BaseModel
 
@@ -21,6 +21,12 @@ class DataStore(CommonModel):
     region: Optional[str]
     endpoint: Optional[str]
     disabled: bool
+    plan: Optional[str]
+    record_count: int
+
+
+class DataStoreInternal(DataStore):
+    stripe_subscription_id: Optional[str]
 
 
 class DataStoreDataResponse(DataResponseModel[DataStore]):
@@ -28,7 +34,7 @@ class DataStoreDataResponse(DataResponseModel[DataStore]):
 
 
 class DataStoreListResponse(PaginationModel[DataStore]):
-	pass
+    pass
 
 
 class DataStoreCreateRequest(BaseModel):
