@@ -1,4 +1,4 @@
-from typing import List, Optional
+from typing import Optional
 from uuid import UUID
 
 from pydantic import BaseModel
@@ -29,7 +29,7 @@ class DataStoreAssociationDataResponse(DataResponseModel[DataStoreAssociation]):
 
 
 class DataStoreAssociationListResponse(PaginationModel[DataStoreAssociation]):
-	pass
+    pass
 
 
 class DataStoreAssociationCreateRequest(BaseModel):
