@@ -7,6 +7,7 @@ from lqs.common.utils import get_logger
 from lqs.transcode.cdr import CDRWrapper
 from lqs.transcode.ark.ark_json_parser import RbufMessageDeserializer
 from lqs.transcode.config import TranscodeConfig
+from lqs.transcode.cutter import CutterDecoder
 from lqs.transcode.ros1.ros_message_deserializer import RosMessageDeserializer
 
 
@@ -23,6 +24,7 @@ class Transcode:
         )
         self.cdr_wrapper = CDRWrapper()
 
+        self.cutter_decoder = CutterDecoder()
         self.logger = get_logger(
             self.__class__.__name__,
             level=self.config.log_level,
@@ -33,7 +35,14 @@ class Transcode:
     def supported_encoding(cls):
         return ["ros1", "rbuf", "json", "cdr"]
 
-    def deserialize(self, type_encoding, type_name, type_data, message_bytes):
+    def deserialize(
+        self,
+        type_encoding,
+        type_name,
+        type_data,
+        message_bytes,
+        replacements=((b"\\u0000", b"NULL"),),
+    ):
         if type_encoding == "ros1":
             message_bytes = io.BytesIO(message_bytes)
             res = self.ros_deserializer.deserialize(
@@ -41,7 +50,10 @@ class Transcode:
                 message_bytes=message_bytes,
                 message_type_data=type_data,
             )
-            return json.loads(json.dumps(res, default=lambda x: x.to_dict()))
+            res_dump = json.dumps(res, default=lambda x: x.to_dict())
+            for replacement in replacements:
+                res_dump = res_dump.replace(*replacement)
+            return json.loads(res_dump)
 
         if type_encoding == "rbuf":
             message_bytes = io.BytesIO(message_bytes)
@@ -50,7 +62,10 @@ class Transcode:
                 message_type=type_name,
                 message_type_data=json.loads(type_data),
             )
-            return json.loads(json.dumps(res, default=lambda x: x.to_dict()))
+            res_dump = json.dumps(res, default=lambda x: x.to_dict())
+            for replacement in replacements:
+                res_dump = res_dump.replace(*replacement)
+            return json.loads(res_dump)
 
         if type_encoding == "json":
             return json.loads(message_bytes)
@@ -62,6 +77,14 @@ class Transcode:
                 message_bytes=message_bytes,
             )
 
+        if not type_encoding and type_name and type_name.startswith("lqs_cutter"):
+            return self.cutter_decoder.deserialize(
+                type_name=type_name, payload=message_bytes
+            )
+
+        if type_encoding is None:
+            return message_bytes
+
         raise NotImplementedError(f"Unknown type encoding: {type_encoding}")
 
     def serialize(self, type_encoding, type_name, type_data, message_dict):
