@@ -1,10 +1,8 @@
 import orjson
 import json
-from rosbags.typesys import get_types_from_msg
 from dataclasses import asdict, is_dataclass
 from rosbags.typesys import (
     Stores,
-    get_types_from_idl,
     get_types_from_msg,
     get_typestore,
 )
@@ -29,7 +27,7 @@ class CDRWrapper:
         self.type_store = get_typestore(Stores.LATEST)
 
     def deserialize(self, type_name, type_data, message_bytes):
-        if type_name not in self.type_store.FIELDDEFS:
+        if type_name not in self.type_store.fielddefs:
             types_dict = get_types_from_msg(text=type_data, name=type_name)
             self.type_store.register(typs=types_dict)
 
